require "behaviours/wander"
require "behaviours/follow"
require "behaviours/faceentity"
require "behaviours/chaseandattack"
require "behaviours/runaway"
require "behaviours/doaction"
require "behaviours/panic"
require "behaviours/leash"

local SEE_DIST = 20

local MAX_WANDER_DIST = 20

local MIN_FOLLOW_LEADER = 0
local MAX_FOLLOW_LEADER = 12
local TARGET_FOLLOW_LEADER = (MAX_FOLLOW_LEADER+MIN_FOLLOW_LEADER)/2


local GO_HOME_DIST = 5


local MAX_CHASE_TIME = 10
local MAX_CHASE_DIST = 30


local RUN_AWAY_DIST = 5
local STOP_RUN_AWAY_DIST = 8


local Pumpkin_LordBrain = Class(Brain, function(self, inst)
    Brain._ctor(self, inst)
end)

function Pumpkin_LordBrain:OnAttacked(attacker)
	if attacker == GetPlayer() then
	self.inst.components.talker:Say("Friendy fire!")
	self.inst.SoundEmitter:PlaySound("dontstarve/characters/wolfgang/grow_medtolrg")
	else
    self.inst.components.combat:SetTarget(attacker)
    self.inst.components.combat:ShareTarget(attacker, 30, function(dude) return dude:HasTag("summonedbyplayer") and dude.components.follower.leader == GetPlayer() end, 5)
	end
end

function Pumpkin_LordBrain:OnAttackOther(target)
    self.inst.components.combat:ShareTarget(target, 30, function(dude) return dude:HasTag("summonedbyplayer") and dude.components.follower.leader == GetPlayer() end, 5)
end


local function GetFaceTargetFn(inst)
    return inst.components.follower.leader
end

local function KeepFaceTargetFn(inst, target)
    return inst.components.follower.leader == target
end

local function GetLeader(inst)
    return inst.components.follower and inst.components.follower.leader
end

local function GetStayPos(inst)
	return inst.components.followersitcommand.locations["currentstaylocation"]
end

local function GetWanderPoint(inst)
	if inst.components.followersitcommand and inst.components.followersitcommand:IsCurrentlyStaying() then
		return GetStayPos(inst)
	else
		local target = GetLeader(inst) or GetPlayer()
   		if target then
        		return target:GetPosition()
    		end
	end
end

local function ShouldGoHome(inst)
    local homePos = inst.components.followersitcommand.locations["currentstaylocation"]
    local myPos = Vector3(inst.Transform:GetWorldPosition() )
    return (homePos and distsq(homePos, myPos) > GO_HOME_DIST*GO_HOME_DIST)
end

local function GoHomeAction(inst)
    local homePos = inst.components.followersitcommand.locations["currentstaylocation"]
    if homePos then
        return BufferedAction(inst, nil, ACTIONS.WALKTO, nil, homePos, nil, 0.2)
    end
end

function Pumpkin_LordBrain:OnStart()
       
    local root = 
        PriorityNode(
        {
		
		
		
	    IfNode(function() 
			local target = self.inst.components.combat.target
		    	if target and target:HasTag("bee") then
				local beehome = target.components.homeseeker and target.components.homeseeker.home 
				if beehome and beehome.prefab == "beebox" then
					self.inst.components.combat:GiveUp()
				end
			end
		end,
		"OnFire", Panic(self.inst) ),
            
     
            
            WhileNode( function() return self.inst.components.combat.target == nil or not self.inst.components.combat:InCooldown() end, "AttackMomentarily",
                    ChaseAndAttack(self.inst, MAX_CHASE_TIME, MAX_CHASE_DIST) ),
            WhileNode( function() return self.inst.components.combat.target and self.inst.components.combat:InCooldown() end, "Dodge",
                    RunAway(self.inst, function() return self.inst.components.combat.target end, RUN_AWAY_DIST, STOP_RUN_AWAY_DIST) ),
         
	    IfNode(function() 
		    if (self.inst.components.follower.leader == nil) then
			    	local x,y,z = self.inst.Transform:GetWorldPosition()
				local ents = TheSim:FindEntities(x,y,z, TUNING.SPIDERHAT_RANGE, {"player"})
				if (#ents > 0) then
					local newfather = ents[1]
					newfather.components.leader:AddFollower(self.inst)
					self.inst.components.follower.maxfollowtime = 99999999
					self.inst.components.follower:AddLoyaltyTime(9999999)
					self.inst.components.combat:GiveUp()
					return true
				else return false
				end
	   		end
   		end,
		"OnFire", Panic(self.inst) ),
	
	IfNode(function() 
		if self.inst.components.follower.leader ~= nil and self.inst.components.followersitcommand and self.inst.components.followersitcommand:IsCurrentlyStaying() == false then
			return true
		elseif self.inst.components.follower.leader ~= nil and not self.inst.components.followersitcommand then
			return true
		end
		end, "has leader",	
		Follow(self.inst, GetLeader, MIN_FOLLOW_LEADER, TARGET_FOLLOW_LEADER, MAX_FOLLOW_LEADER)),
        -- and one where it is "gohome" to a position
	IfNode(function() 
		
		if self.inst.components.follower.leader ~= nil and self.inst.components.followersitcommand and self.inst.components.followersitcommand:IsCurrentlyStaying() == true then
			return true
		end
		end, "has leader",	
		WhileNode(function() return ShouldGoHome(self.inst) end, "ShouldGoHome",
            		DoAction(self.inst, GoHomeAction, "Go Home", true ))),  

	IfNode(function() return self.inst.components.follower.leader ~= nil end, "has leader",
		FaceEntity(self.inst, GetFaceTargetFn, KeepFaceTargetFn )),
	  

	  
        }, .1)
    
    self.bt = BT(self.inst, root)
    
    
    
    
   self.inst:ListenForEvent("attacked", function(inst, data) self:OnAttacked(data.attacker) end)  
   self.inst:ListenForEvent("onattackother", function(inst, data) self:OnAttackOther(data.target) end)
end


return Pumpkin_LordBrain